/*
 * uart_bsp.c
 *
 *  Created on: Jul 11, 2023
 *      Author: hussamaldean
 */



#include "uart_bsp.h"
#include "bsp.h"


static void uart_set_baudrate(USART_TypeDef *USARTx, uint32_t PeriphClk,  uint32_t BaudRate);
static uint16_t compute_uart_bd(uint32_t PeriphClk, uint32_t BaudRate);



void UART2_CLOCK_ENABLE(void)
{
	RCC->APB1ENR|=RCC_APB1ENR_USART2EN;
}

void BSP_UART_Init(USART_TypeDef *uart, UART_ConfigTypedef *config)
{
	switch (config->mode)
	{
		case UART_TX_Only: 	uart->CR1=USART_CR1_TE; break;

		case UART_RX_Only:  uart->CR1=USART_CR1_RE; break;

		case UART_TX_RX:	uart->CR1=USART_CR1_RE|USART_CR1_TE; break;
	}

	uart_set_baudrate(uart, config->busSpeed, config->buadRate);

	uart->CR1|= USART_CR1_UE;

}



UART_StatusTypedef BSP_UART_Write_Char(USART_TypeDef *uart,char ch, uint32_t timeout)
{
	uint32_t start_ticks= BSP_Get_Ticks();

	/*Make sure the transmit data register is empty*/
	while(!(uart->SR & USART_SR_TXE))
	{
		if(BSP_Get_Ticks()-start_ticks>timeout)
		{
			return UART_TIMEOUT;
		}
	}

	/*Write to transmit data register*/
	uart->DR	=  (ch & 0xFF);

	return UART_SUCCESS;
}




static void uart_set_baudrate(USART_TypeDef *USARTx, uint32_t PeriphClk,  uint32_t BaudRate)
{
	USARTx->BRR =  compute_uart_bd(PeriphClk,BaudRate);
}

static uint16_t compute_uart_bd(uint32_t PeriphClk, uint32_t BaudRate)
{
	return ((PeriphClk + (BaudRate/2U))/BaudRate);
}
